import 'package:flutter/material.dart';
import 'package:hometec_ui/components/progress_bar/loading_progress_bar.dart';
import 'package:hometec_ui/components/state_list/model/state_list_model.dart';
import 'package:hometec_ui/measure_size.dart';
import 'package:hometec_ui/resources/colors/color.dart';
import 'package:hometec_ui/resources/fonts/font.dart';
import 'package:rxdart/subjects.dart';

class StateList extends StatefulWidget {

  final Widget child;
  final Stream<StateListEnum> state;
  final String emptyMessage;
  final String errorMessage;
  final String networkMessage;
  final Widget other;
  final bool measure;

  const StateList({Key key, @required this.child, @required this.state, @required this.emptyMessage, @required this.errorMessage, this.other, this.networkMessage, this.measure = true}) : super(key: key);

  @override
  _StateListState createState() => _StateListState(child, state, other);
}

class _StateListState extends State<StateList> {
  final Widget child;
  final Stream<StateListEnum> state;
  final Widget other;
  BehaviorSubject<double> _heightController = BehaviorSubject<double>();

  _StateListState(this.child, this.state, this.other);

  @override
  Widget build(BuildContext context) {
    return MeasureSize(
      onChange: (size) {
        _heightController.add(widget.measure ? size.height : 165);
      },
      child: StreamBuilder(
      stream: state,
      builder: (context, snapshot){
        if(snapshot.hasData)
          switch (snapshot.data) {
            case StateListEnum.success:
              return Container(
                child: child
              );
            case StateListEnum.other:
              return Container(
                child: other
              );
            case StateListEnum.empty:
              return FutureBuilder<dynamic>(
                future: _heightController.done,
                builder: (context, snapshot){
                  return SingleChildScrollView(
                    physics: widget.measure ? AlwaysScrollableScrollPhysics() : NeverScrollableScrollPhysics(),
                    child: Container(
                      height: snapshot.data,
                      child: Padding(
                        padding: EdgeInsets.all(24),
                        child: Center(
                          child: Text(widget.emptyMessage, textAlign: TextAlign.center, style: Font.bodyRegular.style(color: ECColor.textColor),),
                        )
                      ),
                    )
                  );
                }
              );
            case StateListEnum.error:
              return FutureBuilder<dynamic>(
                future: _heightController.done,
                builder: (context, snapshot){
                  return SingleChildScrollView(
                    physics: widget.measure ? AlwaysScrollableScrollPhysics() : NeverScrollableScrollPhysics(),
                    child: Container(
                    height: snapshot.data,
                    child: Padding(
                      padding: EdgeInsets.all(24),
                        child: Center(
                          child: Column(
                            mainAxisSize: MainAxisSize.min,
                            children: <Widget>[
                              Icon(Icons.report_problem, size: 72, color: ECColor.textColor),
                              SizedBox(height: 8),
                              Text(widget.errorMessage, textAlign: TextAlign.center, style: Font.bodyRegular.style(color: ECColor.textColor),),
                            ],
                          ),
                        )
                      ),
                    )
                  );
                }
              );             
            case StateListEnum.network:
              return FutureBuilder<dynamic>(
                future: _heightController.done,
                builder: (context, snapshot){
                  return SingleChildScrollView(
                    physics: widget.measure ? AlwaysScrollableScrollPhysics() : NeverScrollableScrollPhysics(),
                    child: Container(
                    height: snapshot.data,
                    child: Padding(
                      padding: EdgeInsets.all(24),
                        child: Center(
                          child: Column(
                            mainAxisSize: MainAxisSize.min,
                            children: <Widget>[
                              Icon(Icons.signal_wifi_off, size: 72, color: ECColor.textColor),
                              SizedBox(height: 8),
                              Text(widget.networkMessage, textAlign: TextAlign.center, style: Font.bodyRegular.style(color: ECColor.textColor),),
                            ],
                          )
                        )
                      ),
                    )
                  );
                }
              );
            default:
              return Center(
              child: LoadingProgressBar(),
            );
          }
        else
          return Center(
              child: LoadingProgressBar(),
          );
        }
      )
    );
  }

  @override
  void dispose() {
    if(_heightController.isBroadcast)
      _heightController.close();
    super.dispose();
  }
}